from Moebelprojekt.leinwand import *
import tkinter
from abc import ABCMeta,abstractmethod

class Form(metaclass=ABCMeta):
    """
    Dies ist eine allgemeine Klasse, die die gemeinsamen Eigenschaften
    aller Formen beinhaltet.
    """
    item=None
    xPosition=None
    yPosition=None
    farbe=''
    randfarbe=''
    bild=None
    winkel=0

    @abstractmethod
    def zeichne(self,leinwand):
        """
        Zeichnet diese Form auf die Leinwand an. Diese Methode ist von
        jeder einzelnen Form zu implementieren, da die zur Anzeige
        notwendigen Schritte von Form zu Form variieren.
        Dabei liefert sie die ID der Form auf der Leinwand zurück.
        leinwand: die Leinwand, auf die gezeichnet wird
        """
        pass

    def zeige(self):
        """
        Zeigt diese Form auf der Leinwand an.
        """
        if not self.item:
            self.item=self.zeichne(self.bild)

    def verberge(self):
        """
        Entfernt diese Form von der Leinwand. Daraufhin kann sie nicht
        mehr verschoben, gedreht, oder anders manipuliert werden, bis
        sie mit einem Aufruf von zeige() wieder auf der Leinwand erscheint.
        """
        if self.item:
            self.bild.delete(self.item)
            self.item=None

    def drehe(self,winkel):
        """
        Dreht diese Form auf der Leinwand.
        winkel: der Winkel in Grad, um den dieses Objekt im Uhrzeigersinn
                gedreht werden soll. Für Drehungen gegen den Uhrzeigersinn
                sind negative Zahlen zu verwenden.
        """
        self.winkel+=winkel
        self.bild.drehe(self.item,self.xPosition,self.yPosition,winkel)

    def horizontalBewegen(self,entfernung):
        """
        Bewegt diese Form in horizontaler Richtung auf der Leinwand.
        entfernung: die Anzahl an Pixeln, um die das Objekt nach rechts
                    bewegt werden soll. Für Bewegungen nach links sind
                    negative Zahlen zu verwenden.
        """
        if self.item:
            self.bild.bewegen(self.item,entfernung,0)
        self.xPosition+=entfernung

    def vertikalBewegen(self,entfernung):
        """
        Bewegt diese Form in vertikaler Richtung auf der Leinwand.
        entfernung: die Anzahl an Pixeln, um die das Objekt nach unten
                    bewegt werden soll. Für Bewegungen nach oben sind
                    negative Zahlen zu verwenden.
        """
        if self.item:
            self.bild.bewegen(self.item,0,entfernung)
        self.yPosition+=entfernung

    def groesseAendern(self,neueBreite=None,neueHoehe=None):
        """
        Ändert die Größe dieser Form auf der Leinwand.
        neueBreite: die Breite der Form in Pixeln
        neueHoehe: die Höhe der Form in Pixeln
        """
        if neueHoehe is None:
            neueHoehe=self.hoehe
        if neueBreite is None:
            neueBreite=self.breite
        if self.item:
            self.bild.skalieren(self.item,self.xPosition,self.yPosition,\
                            neueBreite/self.breite,neueHoehe/self.hoehe)
        self.breite=neueBreite
        self.hoehe=neueHoehe

    def farbeAendern(self,neueFarbe):
        """
        Ändert die Füllfarbe der Form auf der Leinwand.
        neueFarbe: einer der Farbnamen aus der Klasse Farben 
        """
        if self.item:
            self.bild.farbeAendern(self.item,neueFarbe)
        self.farbe=neueFarbe

    def randfarbeAendern(self,neueFarbe):
        """
        Ändert die Randfarbe der Form auf der Leinwand.
        neueFarbe: einer der Farbnamen aus der Klasse Farben 
        """
        if self.item:
            self.bild.randfarbeAendern(self.item,neueFarbe)
        self.randfarbe=neueFarbe


class Rechteck(Form):
    """
    Stellt eine rechteckige Form auf der Leinwand dar.
    """
    def __init__(self,x=60,y=50,breite=30,hoehe=30,farbe='gruen',randfarbe='schwarz'):
        """
        Erzeugt ein Rechteck an der angegebenen Position.
        x: die x-Koordinate der oberen linken Ecke
        y: die y-Koordinate der oberen linken Ecke
        breite: die Breite des Rechtecks
        hoehe: die Höhe des Rechtecks
        farbe: die Füllfarbe des Rechtecks. Sollte einer der Farbnamen
               aus der Klasse Farben sein
        randfarbe: die Randfarbe des Rechtecks. Sollte einer der Farbnamen
                   aus der Klasse Farben sein
        """
        self.xPosition=x
        self.yPosition=y
        self.breite=breite
        self.hoehe=hoehe
        self.farbe=farbe
        self.randfarbe=randfarbe
        self.bild=Leinwand()
    
    def zeichne(self,leinwand):
        """
        Zeichnet dieses Rechteck auf die Leinwand.
        leinwand: die Leinwand, auf die gezeichnet wird
        """
        return leinwand.polygon(self.xPosition,self.yPosition,\
                                self.xPosition+self.breite,self.yPosition,\
                                self.xPosition+self.breite,self.yPosition+self.hoehe,\
                                self.xPosition,self.yPosition+self.hoehe,\
                                fill=self.bild.farben.gibFarbe(self.farbe),\
                                outline=self.bild.farben.gibFarbe(self.randfarbe))

class Quadrat(Rechteck):
    """
    Stellt eine quadratische Form auf der Leinwand dar.
    """
    def __init__(self,x=60,y=50,groesse=30,farbe='rot',randfarbe='schwarz'):
        """
        Erzeugt ein Quadrat an der angegebenen Position.
        x: die x-Koordinate der oberen linken Ecke
        y: die y-Koordinate der oberen linken Ecke
        groesse: die Breite und Höhe des Quadrats
        farbe: die Füllfarbe des Quadrats. Sollte einer der Farbnamen
               aus der Klasse Farben sein
        randfarbe: die Randfarbe des Quadrats. Sollte einer der Farbnamen
                   aus der Klasse Farben sein
        """
        super().__init__(x,y,groesse,groesse,farbe,randfarbe)

    def groesseAendern(self,neueGroesse):
        """
        Ändert die Breite und Höhe dieses Quadrats auf der Leinwand.
        neueGroesse: die Breite und Höhe des Quadrats in Pixeln
        """
        super().groesseAendern(neueGroesse,neueGroesse)

class Kreis(Form):
    """
    Stellt eine runde Form auf der Leinwand dar.
    """
    def __init__(self,x=20,y=60,durchmesser=30,farbe='blau',randfarbe='schwarz'):
        """
        Erzeugt einen Kreis an der angegebenen Position.
        x: die x-Koordinate des Mittelpunkts
        y: die y-Koordinate des Mittelpunkts
        durchmesser: der Durchmesser des Kreises
        farbe: die Füllfarbe des Kreises. Sollte einer der Farbnamen
               aus der Klasse Farben sein
        randfarbe: die Randfarbe des Kreises. Sollte einer der Farbnamen
                   aus der Klasse Farben sein
        """
        self.xPosition=x
        self.yPosition=y
        self.breite=self.hoehe=durchmesser
        self.farbe=farbe
        self.randfarbe=randfarbe
        self.bild=Leinwand()

    def zeichne(self,leinwand):
        """
        Zeigt diesen Kreis auf der Leinwand an.
        leinwand: die Leinwand, auf die gezeichnet wird
        """
        return leinwand.ellipse(self.xPosition-self.breite/2,self.yPosition-self.hoehe/2,\
                                self.xPosition+self.breite/2,self.yPosition+self.hoehe/2,\
                                fill=self.bild.farben.gibFarbe(self.farbe),\
                                outline=self.bild.farben.gibFarbe(self.randfarbe))

    def groesseAendern(self,neuerDurchmesser):
        """
        Ändert den Durchmesser dieses Kreises auf der Leinwand.
        neuerDurchmesser: der Durchmesser des Kreises in Pixeln
        """
        super().groesseAendern(neuerDurchmesser,neuerDurchmesser)
    
class Dreieck(Form):
    """
    Stellt eine dreieckige Form auf der Leinwand dar.
    """
    def __init__(self,x=20,y=60,breite=60,hoehe=40,farbe='gelb',randfarbe='schwarz'):
        """
        Erzeugt ein gleichschenkliges Dreieck mit waagerechter Grundlinie
        an der angegebenen Position.
        x: die x-Koordinate der unteren linken Ecke
        y: die y-Koordinate der unteren linken Ecke
        breite: die Breite des Dreiecks
        hoehe: die Höhe des Dreiecks
        farbe: die Füllfarbe des Dreiecks. Sollte einer der Farbnamen
               aus der Klasse Farben sein
        randfarbe: die Randfarbe des Dreiecks. Sollte einer der Farbnamen
                   aus der Klasse Farben sein
        """
        self.xPosition=x
        self.yPosition=y
        self.breite=breite
        self.hoehe=hoehe
        self.farbe=farbe
        self.randfarbe=randfarbe
        self.bild=Leinwand()

    def zeichne(self,leinwand):
        """
        Zeichnet dieses Dreieck auf die Leinwand.
        leinwand: die Leinwand, auf die gezeichnet wird
        """
        return leinwand.polygon(self.xPosition,self.yPosition,\
                                self.xPosition+self.breite/2,self.yPosition-self.hoehe,\
                                self.xPosition+self.breite,self.yPosition,\
                                fill=self.bild.farben.gibFarbe(self.farbe),\
                                outline=self.bild.farben.gibFarbe(self.randfarbe))

# Testaufrufe für dieses Modul:
if __name__=='__main__':
    q=Quadrat()
    q.zeige()

    def bPressed():
        q.groesseAendern(100)
        q.drehe(10)
    
    b=tkinter.Button(Leinwand().fenster,command=bPressed,text='Quadrat ändern')
    b.pack()
    p=Quadrat()
    p.horizontalBewegen(100)
    p.vertikalBewegen(100)
    p.groesseAendern(50)
    p.farbeAendern('grau')
    p.zeige()
    k=Kreis()
    k.zeige()

    d=Dreieck()
    d.zeige()
    Leinwand().fenster.mainloop()
